@@ -115,7 +115,7 @@ pydoctor_args = {
             [f'--config={_pydoctor_root}/docs/source/custom_template_demo/pyproject.toml',
               '-qqq' ], # we don't want to hear any warnings from this custom template demo.
     'epydoc_demo': [
-        '--html-output={outdir}/docformat/epytext',
+        '--html-output={outdir}/docformat/epytext_demo',
         '--project-name=pydoctor-epytext-demo',
         '--project-version=1.3.0',
         '--docformat=epytext',
@@ -125,7 +125,7 @@ pydoctor_args = {
         f'{_pydoctor_root}/docs/epytext_demo',
         ] + _common_args,
     'restructuredtext_demo': [
-        '--html-output={outdir}/docformat/restructuredtext',
+        '--html-output={outdir}/docformat/restructuredtext_demo',
         '--project-name=pydoctor-restructuredtext-demo',
         '--project-version=1.0.0',
         '--docformat=restructuredtext',
@@ -135,7 +135,7 @@ pydoctor_args = {
         f'{_pydoctor_root}/docs/restructuredtext_demo',
         ] + _common_args,
     'numpy_demo': [ # no need to pass --docformat here, we use __docformat__
-        '--html-output={outdir}/docformat/numpy',
+        '--html-output={outdir}/docformat/numpy_demo',
         '--project-name=pydoctor-numpy-style-demo',
         '--project-version=1.0.0',
         '--project-url=../google-numpy.html',
@@ -144,7 +144,7 @@ pydoctor_args = {
         f'{_pydoctor_root}/pydoctor/napoleon'
         ] + _common_args,
     'google_demo': [
-        '--html-output={outdir}/docformat/google',
+        '--html-output={outdir}/docformat/google_demo',
         '--project-name=pydoctor-google-style-demo',
         '--project-version=1.0.0',
         '--docformat=google',
