@@ -2,9 +2,11 @@
 Various bits of reusable code related to L{ast.AST} node processing.
 """
 
+import inspect
+import platform
 import sys
 from numbers import Number
-from typing import Iterator, Optional, List, Iterable, Sequence, TYPE_CHECKING, Union
+from typing import Iterator, Optional, List, Iterable, Sequence, TYPE_CHECKING, Tuple, Union, cast
 from inspect import BoundArguments, Signature
 import ast
 
@@ -146,6 +148,12 @@ else:
     def _is_str_constant(expr: ast.expr, s: str) -> bool:
         return isinstance(expr, ast.Str) and expr.s == s
 
+def get_int_value(expr: ast.expr) -> Optional[int]:
+    num = get_num_value(expr)
+    if isinstance(num, int):
+        return num # type:ignore[unreachable]
+    return None
+
 def is__name__equals__main__(cmp: ast.Compare) -> bool:
     """
     Returns whether or not the given L{ast.Compare} is equal to C{__name__ == '__main__'}.
@@ -183,7 +191,11 @@ def is_using_annotations(expr: Optional[ast.AST],
                 return True
     return False
 
-def unstring_annotation(node: ast.expr, ctx:'model.Documentable') -> ast.expr:
+def is_none_literal(node: ast.expr) -> bool:
+    """Does this AST node represent the literal constant None?"""
+    return isinstance(node, (ast.Constant, ast.NameConstant)) and node.value is None
+    
+def unstring_annotation(node: ast.expr, ctx:'model.Documentable', section:str='annotation') -> ast.expr:
     """Replace all strings in the given expression by parsed versions.
     @return: The unstringed node. If parsing fails, an error is logged
         and the original node is returned.
@@ -193,7 +205,7 @@ def unstring_annotation(node: ast.expr, ctx:'model.Documentable') -> ast.expr:
     except SyntaxError as ex:
         module = ctx.module
         assert module is not None
-        module.report(f'syntax error in annotation: {ex}', lineno_offset=node.lineno)
+        module.report(f'syntax error in {section}: {ex}', lineno_offset=node.lineno, section=section)
         return node
     else:
         assert isinstance(expr, ast.expr), expr
@@ -345,3 +357,130 @@ def is_typing_annotation(node: ast.AST, ctx: 'model.Documentable') -> bool:
     return is_using_annotations(node, TYPING_ALIAS, ctx) or \
             is_using_annotations(node, SUBSCRIPTABLE_CLASSES_PEP585, ctx)
 
+
+_string_lineno_is_end = sys.version_info < (3,8) \
+                    and platform.python_implementation() != 'PyPy'
+"""True iff the 'lineno' attribute of an AST string node points to the last
+line in the string, rather than the first line.
+"""
+
+def extract_docstring_linenum(node: ast.Str) -> int:
+    r"""
+    In older CPython versions, the AST only tells us the end line
+    number and we must approximate the start line number.
+    This approximation is correct if the docstring does not contain
+    explicit newlines ('\n') or joined lines ('\' at end of line).
+
+    Leading blank lines are stripped by cleandoc(), so we must
+    return the line number of the first non-blank line.
+    """
+    doc = node.s
+    lineno = node.lineno
+    if _string_lineno_is_end:
+        # In older CPython versions, the AST only tells us the end line
+        # number and we must approximate the start line number.
+        # This approximation is correct if the docstring does not contain
+        # explicit newlines ('\n') or joined lines ('\' at end of line).
+        lineno -= doc.count('\n')
+
+    # Leading blank lines are stripped by cleandoc(), so we must
+    # return the line number of the first non-blank line.
+    for ch in doc:
+        if ch == '\n':
+            lineno += 1
+        elif not ch.isspace():
+            break
+    
+    return lineno
+
+def extract_docstring(node: ast.Str) -> Tuple[int, str]:
+    """
+    Extract docstring information from an ast node that represents the docstring.
+
+    @returns: 
+        - The line number of the first non-blank line of the docsring. See L{extract_docstring_linenum}.
+        - The docstring to be parsed, cleaned by L{inspect.cleandoc}.
+    """
+    lineno = extract_docstring_linenum(node)
+    return lineno, inspect.cleandoc(node.s)
+
+
+def infer_type(expr: ast.expr) -> Optional[ast.expr]:
+    """Infer a literal expression's type.
+    @param expr: The expression's AST.
+    @return: A type annotation, or None if the expression has no obvious type.
+    """
+    try:
+        value: object = ast.literal_eval(expr)
+    except (ValueError, TypeError):
+        return None
+    else:
+        ann = _annotation_for_value(value)
+        if ann is None:
+            return None
+        else:
+            return ast.fix_missing_locations(ast.copy_location(ann, expr))
+
+def _annotation_for_value(value: object) -> Optional[ast.expr]:
+    if value is None:
+        return None
+    name = type(value).__name__
+    if isinstance(value, (dict, list, set, tuple)):
+        ann_elem = _annotation_for_elements(value)
+        if isinstance(value, dict):
+            ann_value = _annotation_for_elements(value.values())
+            if ann_value is None:
+                ann_elem = None
+            elif ann_elem is not None:
+                ann_elem = ast.Tuple(elts=[ann_elem, ann_value])
+        if ann_elem is not None:
+            if name == 'tuple':
+                ann_elem = ast.Tuple(elts=[ann_elem, ast.Ellipsis()])
+            return ast.Subscript(value=ast.Name(id=name),
+                                 slice=ast.Index(value=ann_elem))
+    return ast.Name(id=name)
+
+def _annotation_for_elements(sequence: Iterable[object]) -> Optional[ast.expr]:
+    names = set()
+    for elem in sequence:
+        ann = _annotation_for_value(elem)
+        if isinstance(ann, ast.Name):
+            names.add(ann.id)
+        else:
+            # Nested sequences are too complex.
+            return None
+    if len(names) == 1:
+        name = names.pop()
+        return ast.Name(id=name)
+    else:
+        # Empty sequence or no uniform type.
+        return None
+
+      
+class Parentage(ast.NodeTransformer):
+    """
+    Add C{parent} attribute to ast nodes instances.
+    """
+    # stolen from https://stackoverflow.com/a/68845448
+    parent: Optional[ast.AST] = None
+
+    def visit(self, node: ast.AST) -> ast.AST:
+        setattr(node, 'parent', self.parent)
+        self.parent = node
+        node = super().visit(node)
+        if isinstance(node, ast.AST):
+            self.parent = getattr(node, 'parent')
+        return node
+
+def get_parents(node:ast.AST) -> Iterator[ast.AST]:
+    """
+    Once nodes have the C{.parent} attribute with {Parentage}, use this function
+    to get a iterator on all parents of the given node up to the root module.
+    """
+    def _yield_parents(n:Optional[ast.AST]) -> Iterator[ast.AST]:
+        if n:
+            yield n
+            p = cast(ast.AST, getattr(n, 'parent', None))
+            yield from _yield_parents(p)
+    yield from _yield_parents(getattr(node, 'parent', None))
+
