@@ -3,11 +3,32 @@ Collection of helper functions and classes related to the creation and processin
 """
 from typing import Iterable, Iterator, Optional
 
-from docutils import nodes
+import optparse
+
+from docutils import nodes, utils, frontend, __version_info__ as docutils_version_info
 from docutils.transforms import parts
 
 __docformat__ = 'epytext en'
 
+_DEFAULT_DOCUTILS_SETTINGS: Optional[optparse.Values] = None
+
+def new_document(source_path: str, settings: Optional[optparse.Values] = None) -> nodes.document:
+    """
+    Create a new L{nodes.document} using the provided settings or cached default settings.
+
+    @returns: L{nodes.document}
+    """
+    global _DEFAULT_DOCUTILS_SETTINGS
+    # If we have docutils >= 0.19 we use get_default_settings to calculate and cache
+    # the default settings. Otherwise we let new_document figure it out.
+    if settings is None and docutils_version_info >= (0,19):
+        if _DEFAULT_DOCUTILS_SETTINGS is None:
+            _DEFAULT_DOCUTILS_SETTINGS = frontend.get_default_settings() # type:ignore[attr-defined]
+
+        settings = _DEFAULT_DOCUTILS_SETTINGS
+
+    return utils.new_document(source_path, settings)
+
 def _set_nodes_parent(nodes: Iterable[nodes.Node], parent: nodes.Element) -> Iterator[nodes.Node]:
     """
     Set the L{nodes.Node.parent} attribute of the C{nodes} to the defined C{parent}. 
