@@ -40,11 +40,11 @@ import re
 from importlib import import_module
 from inspect import getmodulename
 
-from docutils import nodes, utils
+from docutils import nodes
 from twisted.web.template import Tag, tags
 
 from pydoctor import node2stan
-from pydoctor.epydoc.docutils import set_node_attributes, build_table_of_content
+from pydoctor.epydoc.docutils import set_node_attributes, build_table_of_content, new_document
 
 
 # In newer Python versions, use importlib.resources from the standard library.
@@ -57,6 +57,9 @@ else:
 if TYPE_CHECKING:
     from twisted.web.template import Flattenable
     from pydoctor.model import Documentable
+    from typing import Protocol
+else:
+    Protocol = object
 
 ##################################################
 ## Contents
@@ -68,6 +71,8 @@ if TYPE_CHECKING:
 # 4. ParseError exceptions
 #
 
+ParserFunction = Callable[[str, List['ParseError']], 'ParsedDocstring']
+
 def get_supported_docformats() -> Iterator[str]:
     """
     Get the list of currently supported docformat.
@@ -79,7 +84,7 @@ def get_supported_docformats() -> Iterator[str]:
         else:
             yield moduleName
 
-def get_parser_by_name(docformat: str, obj: Optional['Documentable'] = None) -> Callable[[str, List['ParseError'], bool], 'ParsedDocstring']:
+def get_parser_by_name(docformat: str, obj: Optional['Documentable'] = None) -> ParserFunction:
     """
     Get the C{parse_docstring(str, List[ParseError], bool) -> ParsedDocstring} function based on a parser name. 
 
@@ -90,6 +95,30 @@ def get_parser_by_name(docformat: str, obj: Optional['Documentable'] = None) ->
     # We can safely ignore this mypy warning, since we can be sure the 'get_parser' function exist and is "correct".
     return mod.get_parser(obj) # type:ignore[no-any-return]
 
+def processtypes(parse:ParserFunction) -> ParserFunction:
+    """
+    Wraps a docstring parser function to provide option --process-types.
+    """
+    
+    def _processtypes(doc: 'ParsedDocstring', errs: List['ParseError']) -> None:
+        """
+        Mutates the type fields of the given parsed docstring to replace 
+        their body by parsed version with type auto-linking.
+        """
+        from pydoctor.epydoc.markup._types import ParsedTypeDocstring
+        for field in doc.fields:
+            if field.tag() in ParsedTypeDocstring.FIELDS:
+                body = ParsedTypeDocstring(field.body().to_node(), lineno=field.lineno)
+                append_warnings(body.warnings, errs, lineno=field.lineno+1)
+                field.replace_body(body)
+    
+    def parse_and_processtypes(doc:str, errs:List['ParseError']) -> 'ParsedDocstring':
+        parsed_doc = parse(doc, errs)
+        _processtypes(parsed_doc, errs)
+        return parsed_doc
+
+    return parse_and_processtypes
+
 ##################################################
 ## ParsedDocstring
 ##################################################
@@ -117,7 +146,6 @@ class ParsedDocstring(abc.ABC):
 
         self._stan: Optional[Tag] = None
         self._summary: Optional['ParsedDocstring'] = None
-        self._compact = True
 
     @abc.abstractproperty
     def has_body(self) -> bool:
@@ -137,7 +165,7 @@ class ParsedDocstring(abc.ABC):
         except NotImplementedError:
             return None
         contents = build_table_of_content(document, depth=depth)
-        docstring_toc = utils.new_document('toc')
+        docstring_toc = new_document('toc')
         if contents:
             docstring_toc.extend(contents)
             from pydoctor.epydoc.markup.restructuredtext import ParsedRstDocstring
@@ -145,7 +173,7 @@ class ParsedDocstring(abc.ABC):
         else:
             return None
 
-    def to_stan(self, docstring_linker: 'DocstringLinker', compact:bool=True) -> Tag:
+    def to_stan(self, docstring_linker: 'DocstringLinker') -> Tag:
         """
         Translate this docstring to a Stan tree.
 
@@ -158,21 +186,9 @@ class ParsedDocstring(abc.ABC):
         @raises Exception: If something went wrong. Callers should generally catch C{Exception}
             when calling L{to_stan()}.
         """
-        # The following three lines is a hack in order to still show p tags 
-        # around docstrings content when there is only a single line text
-        # and arguement compact=False is passed. We clear cached stan if required.
-        if compact != self._compact and self._stan is not None:
-            self._stan = None
-        self._compact = compact
-
         if self._stan is not None:
-            return self._stan      
-
-        docstring_stan = node2stan.node2stan(self.to_node(), 
-                                        docstring_linker, 
-                                        compact=compact)
-
-        self._stan = Tag('', children=docstring_stan.children)
+            return self._stan
+        self._stan = Tag('', children=node2stan.node2stan(self.to_node(), docstring_linker).children)
         return self._stan
     
     @abc.abstractmethod
@@ -250,6 +266,9 @@ class Field:
         @return: This field's body.
         """
         return self._body
+    
+    def replace_body(self, newbody:ParsedDocstring) -> None:
+        self._body = newbody
 
     def __repr__(self) -> str:
         if self._arg is None:
@@ -260,7 +279,7 @@ class Field:
 ##################################################
 ## Docstring Linker (resolves crossreferences)
 ##################################################
-class DocstringLinker:
+class DocstringLinker(Protocol):
     """
     A resolver for crossreference links out of a C{ParsedDocstring}.
     C{DocstringLinker} is used by C{ParsedDocstring} to look up the
@@ -277,7 +296,6 @@ class DocstringLinker:
         @param label: The label to show for the link.
         @return: The link, or just the label if the target was not found.
         """
-        raise NotImplementedError()
 
     def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
         """
@@ -293,28 +311,19 @@ class DocstringLinker:
         @return: The link, or just the label if the target was not found.
             In either case, the returned top-level tag will be C{<code>}.
         """
-        raise NotImplementedError()
 
-    def resolve_identifier(self, identifier: str) -> Optional[str]:
+    def switch_context(self, ob:Optional['Documentable']) -> ContextManager[None]:
         """
-        Resolve a Python identifier.
-        This will resolve the identifier like Python itself would.
+        Switch the context of the linker, keeping the same underlying lookup rules.
 
-        @param identifier: The name of the Python identifier that
-            should be linked to.
-        @return: The URL of the target, or L{None} if not found.
-        """
-        raise NotImplementedError()
-    
-    def disable_same_page_optimazation(self) -> ContextManager[None]:
-        """
-        By default, when linkng to an object on the same page, the linker will generate 
-        an URL that links to the anchor only, this will avoid reloading the page needlessly. But sometimes 
-        we're using a linker to present the content on another page. This context manager will 
-        make the linker always generate full URLs.
-        """
-        raise NotImplementedError()
+        Useful to resolve links with the right L{Documentable} context but
+        create correct - absolute or relative - links to be clicked on from another page 
+        rather than the initial page of the context. "Cannot find link target" errors will be reported
+        relatively to the new context object.
 
+        Pass C{None} to always generate full URLs (for summaries for example), 
+        in this case error will NOT be reported at all.
+        """
 
 ##################################################
 ## ParseError exceptions
@@ -427,7 +436,7 @@ class SummaryExtractor(nodes.NodeVisitor):
             self.other_docs = True
             raise nodes.StopTraversal()
 
-        summary_doc = utils.new_document('summary')
+        summary_doc = new_document('summary')
         summary_pieces = []
 
         # Extract the first sentences from the first paragraph until maximum number 
