@@ -3,9 +3,10 @@ Convert L{pydoctor.epydoc} parsed markup into renderable content.
 """
 
 from collections import defaultdict
+import enum
 from typing import (
-    TYPE_CHECKING, Any, Callable, ClassVar, DefaultDict, Dict, Generator, 
-    Iterator, List, Mapping, Optional, Sequence, Tuple, 
+    TYPE_CHECKING, Any, Callable, ClassVar, DefaultDict, Dict, Generator,
+    Iterator, List, Mapping, Optional, Sequence, Tuple, Union,
 )
 import ast
 import re
@@ -13,7 +14,8 @@ import re
 import attr
 
 from pydoctor import model, linker, node2stan
-from pydoctor.epydoc.markup import Field as EpydocField, ParseError, get_parser_by_name
+from pydoctor.astutils import is_none_literal
+from pydoctor.epydoc.markup import Field as EpydocField, ParseError, get_parser_by_name, processtypes
 from twisted.web.template import Tag, tags
 from pydoctor.epydoc.markup import ParsedDocstring, DocstringLinker
 import pydoctor.epydoc.markup.plaintext
@@ -29,48 +31,19 @@ Alias to L{pydoctor.linker.taglink()}.
 
 BROKEN = tags.p(class_="undocumented")('Broken description')
 
-def get_parser(obj: model.Documentable) -> Callable[[str, List[ParseError], bool], ParsedDocstring]:
+def _get_docformat(obj: model.Documentable) -> str:
     """
-    Get the C{parse_docstring(str, List[ParseError], bool) -> ParsedDocstring} function. 
-    """    
-    # Use module's __docformat__ if specified, else use system's. 
+    Returns the docformat to use to parse the docstring of this object.
+    """
+    # Use module's __docformat__ if specified, else use system's.
     # Except if system's docformat is plaintext, in this case, use plaintext.
     # See https://github.com/twisted/pydoctor/issues/503 for the reason
-    # of this behavior. 
+    # of this behavior.
     if obj.system.options.docformat == 'plaintext':
-        return pydoctor.epydoc.markup.plaintext.parse_docstring
+        return 'plaintext'
     # the docstring should be parsed using the format of the module it was inherited from
     docformat = obj.module.docformat or obj.system.options.docformat
-    
-    try:
-        return get_parser_by_name(docformat, obj)
-    except ImportError as e:
-        msg = 'Error trying to import %r parser:\n\n    %s: %s\n\nUsing plain text formatting only.'%(
-            docformat, e.__class__.__name__, e)
-        obj.system.msg('epydoc2stan', msg, thresh=-1, once=True)
-        return pydoctor.epydoc.markup.plaintext.parse_docstring
-
-
-def get_docstring(
-        obj: model.Documentable
-        ) -> Tuple[Optional[str], Optional[model.Documentable]]:
-    """
-    Fetch the docstring for a documentable. 
-    Treat empty docstring as undocumented.
-
-    :returns: 
-        - C{(docstring, source)} if the object is documented.
-        - C{(None, None)} if the object has no docstring (even inherited).
-        - C{(None, source)} if the object has an empty docstring.
-    """
-    for source in obj.docsources():
-        doc = source.docstring
-        if doc:
-            return doc, source
-        if doc is not None:
-            # Treat empty docstring as undocumented.
-            return None, source
-    return None, None
+    return docformat
 
 @attr.s(auto_attribs=True)
 class FieldDesc:
@@ -125,6 +98,22 @@ class FieldDesc:
             #  <desc>
             yield tags.td(formatted, colspan="2")
 
+@attr.s(auto_attribs=True)
+class _SignatureDesc(FieldDesc):
+    type_origin: Optional['FieldOrigin'] = None
+
+    def is_documented(self) -> bool:
+        return bool(self.body or self.type_origin is FieldOrigin.FROM_DOCSTRING)
+
+@attr.s(auto_attribs=True)
+class ReturnDesc(_SignatureDesc):...
+
+@attr.s(auto_attribs=True)
+class ParamDesc(_SignatureDesc):...
+
+@attr.s(auto_attribs=True)
+class KeywordDesc(_SignatureDesc):...
+
 class RaisesDesc(FieldDesc):
     """Description of an exception that can be raised by function/method."""
 
@@ -203,7 +192,7 @@ class Field:
 
     def format(self) -> Tag:
         """Present this field's body as HTML."""
-        return safe_to_stan(self.body, self.source.docstring_linker, self.source, compact=True, 
+        return safe_to_stan(self.body, self.source.docstring_linker, self.source,
                     # the parsed docstring maybe doesn't support to_node(), i.e. ParsedTypeDocstring,
                     # so we can only show the broken text.
                     fallback=lambda _, __, ___:BROKEN)
@@ -240,27 +229,36 @@ def format_field_list(singular: str, plural: str, fields: Sequence[Field]) -> It
 
 class VariableArgument(str):
     """
-    Encapsulate the name of C{vararg} parameters.
+    Encapsulate the name of C{vararg} parameters in L{Function.annotations} mapping keys.
     """
 
 class KeywordArgument(str):
     """
-    Encapsulate the name of C{kwarg} parameters.
+    Encapsulate the name of C{kwarg} parameters in L{Function.annotations} mapping keys.
     """
 
+class FieldOrigin(enum.Enum):
+    FROM_AST = 0
+    FROM_DOCSTRING = 1
+
+@attr.s(auto_attribs=True)
+class ParamType:
+    stan: Tag
+    origin: FieldOrigin
+
 class FieldHandler:
 
     def __init__(self, obj: model.Documentable):
         self.obj = obj
         self._linker = self.obj.docstring_linker
 
-        self.types: Dict[str, Optional[Tag]] = {}
+        self.types: Dict[str, Optional[ParamType]] = {}
 
-        self.parameter_descs: List[FieldDesc] = []
-        self.return_desc: Optional[FieldDesc] = None
-        self.yields_desc: Optional[FieldDesc] = None 
+        self.parameter_descs: List[Union[ParamDesc, KeywordDesc]] = []
+        self.return_desc: Optional[ReturnDesc] = None
+        self.yields_desc: Optional[FieldDesc] = None
         self.raise_descs: List[RaisesDesc] = []
-        self.warns_desc: List[FieldDesc] = [] 
+        self.warns_desc: List[FieldDesc] = []
         self.seealsos: List[Field] = []
         self.notes: List[Field] = []
         self.authors: List[Field] = []
@@ -270,12 +268,17 @@ class FieldHandler:
     def set_param_types_from_annotations(
             self, annotations: Mapping[str, Optional[ast.expr]]
             ) -> None:
+        _linker = linker._AnnotationLinker(self.obj)
         formatted_annotations = {
             name: None if value is None
-                       else safe_to_stan(colorize_inline_pyval(value), self.obj.docstring_linker, 
-                                self.obj, compact=True, fallback=colorized_pyval_fallback, section='annotation')
+                       else ParamType(safe_to_stan(colorize_inline_pyval(value), _linker,
+                                self.obj, fallback=colorized_pyval_fallback, section='annotation', report=False),
+                                # don't spam the log, invalid annotation are going to be reported when the signature gets colorized
+                                origin=FieldOrigin.FROM_AST)
+
             for name, value in annotations.items()
             }
+
         ret_type = formatted_annotations.pop('return', None)
         self.types.update(formatted_annotations)
         if ret_type is not None:
@@ -284,8 +287,8 @@ class FieldHandler:
             # it from being presented.
             ann_ret = annotations['return']
             assert ann_ret is not None  # ret_type would be None otherwise
-            if not _is_none_literal(ann_ret):
-                self.return_desc = FieldDesc(type=ret_type)
+            if not is_none_literal(ann_ret):
+                self.return_desc = ReturnDesc(type=ret_type.stan, type_origin=ret_type.origin)
 
     @staticmethod
     def _report_unexpected_argument(field:Field) -> None:
@@ -295,7 +298,7 @@ class FieldHandler:
     def handle_return(self, field: Field) -> None:
         self._report_unexpected_argument(field)
         if not self.return_desc:
-            self.return_desc = FieldDesc()
+            self.return_desc = ReturnDesc()
         self.return_desc.body = field.format()
     handle_returns = handle_return
 
@@ -309,8 +312,9 @@ class FieldHandler:
     def handle_returntype(self, field: Field) -> None:
         self._report_unexpected_argument(field)
         if not self.return_desc:
-            self.return_desc = FieldDesc()
+            self.return_desc = ReturnDesc()
         self.return_desc.type = field.format()
+        self.return_desc.type_origin = FieldOrigin.FROM_DOCSTRING
     handle_rtype = handle_returntype
 
     def handle_yieldtype(self, field: Field) -> None:
@@ -321,11 +325,15 @@ class FieldHandler:
     handle_ytype = handle_yieldtype
 
     def _handle_param_name(self, field: Field) -> Optional[str]:
+        """
+        Returns the Field name and trigger a few warnings for a few scenarios.
+        Note that the return type could be L{VariableArgument} or L{KeywordArgument} or L{str}.
+        """
         name = field.arg
         if name is None:
             field.report('Parameter name missing')
             return None
-        
+
         name = name.lstrip('*')
         annotations = None
         if isinstance(field.source, model.Function):
@@ -333,8 +341,8 @@ class FieldHandler:
         elif isinstance(field.source, model.Class):
             # Constructor parameters can be documented on the class.
             annotations = field.source.constructor_params
-        # This might look useless, but it's needed in order to keep the 
-        # right str type: str, VariableArgument or KeyowrdArgument. And then add the stars accordingly.
+        # This might look useless, but it's needed in order to keep the
+        # right return type and then add the stars accordingly.
         if annotations is not None:
             for param_name, _ in annotations.items():
                 if param_name == name:
@@ -359,7 +367,14 @@ class FieldHandler:
             if name in source.constructor_params:
                 # Constructor parameters can be documented on the class.
                 return
-        field.report('Documented parameter "%s" does not exist' % (name,))
+        msg = f'Documented parameter "{name}" does not exist'
+        if any(isinstance(n, KeywordArgument) for n in self.types):
+            msg += ', variable keywords should be documented with the '
+            if _get_docformat(self.obj) in ('google', 'numpy'):
+                msg += '"Keyword Arguments" section'
+            else:
+                msg += '"keyword" field'
+        field.report(msg)
 
     def handle_type(self, field: Field) -> None:
         if isinstance(self.obj, model.Attribute):
@@ -383,14 +398,14 @@ class FieldHandler:
             #       inconsistencies.
             name = field.arg
         if name is not None:
-            self.types[name] = field.format()
+            self.types[name] = ParamType(field.format(), origin=FieldOrigin.FROM_DOCSTRING)
 
     def handle_param(self, field: Field) -> None:
         name = self._handle_param_name(field)
         if name is not None:
             if any(desc.name == name for desc in self.parameter_descs):
                 field.report('Parameter "%s" was already documented' % (name,))
-            self.parameter_descs.append(FieldDesc(name=name, body=field.format()))
+            self.parameter_descs.append(ParamDesc(name=name, body=field.format()))
             if name not in self.types:
                 self._handle_param_not_found(name, field)
 
@@ -400,7 +415,7 @@ class FieldHandler:
         name = self._handle_param_name(field)
         if name is not None:
             # TODO: How should this be matched to the type annotation?
-            self.parameter_descs.append(FieldDesc(name=name, body=field.format()))
+            self.parameter_descs.append(KeywordDesc(name=name, body=field.format()))
             if name in self.types:
                 field.report('Parameter "%s" is documented as keyword' % (name,))
 
@@ -424,7 +439,7 @@ class FieldHandler:
     handle_raise = handle_raises
     handle_except = handle_raises
 
-    # Warns is just like raises but the syntax is more relax i.e. warning type not required. 
+    # Warns is just like raises but the syntax is more relax i.e. warning type not required.
     def handle_warns(self, field: Field) -> None:
         if field.arg is None:
             typ_fmt = None
@@ -433,7 +448,7 @@ class FieldHandler:
         self.warns_desc.append(FieldDesc(type=typ_fmt, body=field.format()))
 
     handle_warn = handle_warns
-    
+
     def handle_seealso(self, field: Field) -> None:
         self.seealsos.append(field)
     handle_see = handle_seealso
@@ -457,46 +472,75 @@ class FieldHandler:
         m(field)
 
     def resolve_types(self) -> None:
-        """Merge information from 'param' fields and AST analysis."""
+        """Merge information from 'param'/'keyword' fields and AST analysis."""
 
         params = {param.name: param for param in self.parameter_descs}
         any_info = bool(params)
 
         # We create a new parameter_descs list to ensure the parameter order
         # matches the AST order.
-        new_parameter_descs = []
-        for index, (name, type_doc) in enumerate(self.types.items()):
+        new_parameter_descs: List[Union[ParamDesc, KeywordDesc]] = []
+        for index, (name, param_type) in enumerate(self.types.items()):
             try:
                 param = params.pop(name)
             except KeyError:
+                # parameter is not documented with @param.
+
                 if index == 0:
                     # Strip 'self' or 'cls' from parameter table when it semantically makes sens.
                     if name=='self' and self.obj.kind is model.DocumentableKind.METHOD:
                         continue
                     if name=='cls' and self.obj.kind is model.DocumentableKind.CLASS_METHOD:
                         continue
-                    
-                param = FieldDesc(name=name, type=type_doc)
-                any_info |= type_doc is not None
+
+                param = ParamDesc(name=name,
+                    type=param_type.stan if param_type else None,
+                    type_origin=param_type.origin if param_type else None,)
+
+                any_info |= param_type is not None
             else:
-                param.type = type_doc
+                param.type = param_type.stan if param_type else None
+                param.type_origin = param_type.origin if param_type else None
+
             new_parameter_descs.append(param)
 
         # Add any leftover parameters, which includes documented **kwargs keywords
         # and non-existing (but documented) parameters.
         new_parameter_descs += params.values()
 
-        # Only replace the descriptions if at least one parameter is documented
+        # Only update the descriptions if at least one parameter is documented
         # or annotated.
         if any_info:
             self.parameter_descs = new_parameter_descs
 
+        # loops thought the parameters and remove eventual **kwargs 
+        # entry if keywords are specifically documented.
+        kwargs = None
+        has_keywords = False
+        for p in self.parameter_descs:
+            if isinstance(p.name, KeywordArgument):
+                kwargs = p
+                continue
+            if isinstance(p, KeywordDesc):
+                has_keywords = True
+        if kwargs:
+            self.parameter_descs.remove(kwargs)
+            if not has_keywords or kwargs.is_documented():
+                # make sure **kwargs row is presented last in the parameter table
+                self.parameter_descs.append(kwargs)
+
     def format(self) -> Tag:
         r: List[Tag] = []
 
-        r += format_desc_list('Parameters', self.parameter_descs)
-        if self.return_desc:
+        # Only include parameter or return sections if any are documented or any type are documented from @type fields.
+        include_params = False
+        if any(p.is_documented() for p in self.parameter_descs):
+            r += format_desc_list('Parameters', self.parameter_descs)
+            include_params = True
+
+        if self.return_desc and (include_params or self.return_desc.is_documented()):
             r += format_desc_list('Returns', [self.return_desc])
+
         if self.yields_desc:
             r += format_desc_list('Yields', [self.yields_desc])
 
@@ -515,11 +559,6 @@ class FieldHandler:
         else:
             return tags.transparent
 
-
-def _is_none_literal(node: ast.expr) -> bool:
-    """Does this AST node represent the literal constant None?"""
-    return isinstance(node, (ast.Constant, ast.NameConstant)) and node.value is None
-
 def reportWarnings(obj: model.Documentable, warns: Sequence[str], **kwargs:Any) -> None:
     for message in warns:
         obj.report(message, **kwargs)
@@ -527,12 +566,12 @@ def reportWarnings(obj: model.Documentable, warns: Sequence[str], **kwargs:Any)
 def reportErrors(obj: model.Documentable, errs: Sequence[ParseError], section:str='docstring') -> None:
     if not errs:
         return
-    
+
     errors = obj.system.parse_errors[section]
 
     if obj.fullName() not in errors:
         errors.add(obj.fullName())
-        
+
         for err in errs:
             obj.report(
                 f'bad {section}: ' + err.descr(),
@@ -540,7 +579,7 @@ def reportErrors(obj: model.Documentable, errs: Sequence[ParseError], section:st
                 section=section
                 )
 
-
+_docformat_skip_processtypes = ('google', 'numpy', 'plaintext')
 def parse_docstring(
         obj: model.Documentable,
         doc: str,
@@ -558,10 +597,31 @@ def parse_docstring(
     @param section: A custom section to use.
     """
 
-    parser = get_parser(source) if not markup else get_parser_by_name(markup, obj)
+    docformat = _get_docformat(source) if not markup else markup
+
+    # fetch the parser function
+    try:
+        parser = get_parser_by_name(docformat, obj)
+    except ImportError as e:
+        _err = 'Error trying to import %r parser:\n\n    %s: %s\n\nUsing plain text formatting only.'%(
+            docformat, e.__class__.__name__, e)
+        obj.system.msg('epydoc2stan', _err, thresh=-1, once=True)
+        parser = pydoctor.epydoc.markup.plaintext.parse_docstring
+
+    # type processing is always enabled for google and numpy docformat,
+    # it's already part of the specification, doing it now would process types twice.
+    if obj.system.options.processtypes and docformat not in _docformat_skip_processtypes:
+        # This allows epytext and restructuredtext markup to use TypeDocstring as well with a CLI option: --process-types.
+        # It's still technically part of the parsing process, so we use a wrapper function.
+        parser = processtypes(parser)
+
     errs: List[ParseError] = []
     try:
-        parsed_doc = parser(doc, errs, obj.system.options.processtypes)
+        # parse docstring
+        parsed_doc = parser(doc, errs)
+    except ParseError:
+        # this error should already by stored in the errs list
+        parsed_doc = pydoctor.epydoc.markup.plaintext.parse_docstring(doc, errs)
     except Exception as e:
         errs.append(ParseError(f'{e.__class__.__name__}: {e}', 1))
         parsed_doc = pydoctor.epydoc.markup.plaintext.parse_docstring(doc, errs)
@@ -573,16 +633,16 @@ def ensure_parsed_docstring(obj: model.Documentable) -> Optional[model.Documenta
     """
     Currently, it's not 100% clear at what point the L{Documentable.parsed_docstring} attribute is set.
     It can be set from the ast builder or later processing step.
-    
-    This function ensures that the C{parsed_docstring} attribute of a documentable is set to it's final value. 
 
-    @returns: 
-        - If the C{obj.parsed_docstring} is set to a L{ParsedDocstring} instance: 
-          The source object of the docstring (might be different 
+    This function ensures that the C{parsed_docstring} attribute of a documentable is set to it's final value.
+
+    @returns:
+        - If the C{obj.parsed_docstring} is set to a L{ParsedDocstring} instance:
+          The source object of the docstring (might be different
           from C{obj} if the documentation is inherited).
         - If the object is undocumented: C{None}.
     """
-    doc, source = get_docstring(obj)
+    doc, source = model.get_docstring(obj)
 
     # Use cached or split version if possible.
     parsed_doc = obj.parsed_docstring
@@ -599,7 +659,7 @@ def ensure_parsed_docstring(obj: model.Documentable) -> Optional[model.Documenta
         assert source is not None
         parsed_doc = parse_docstring(obj, doc, source)
         obj.parsed_docstring = parsed_doc
-    
+
     if obj.parsed_docstring is not None:
         return source
     else:
@@ -609,28 +669,32 @@ def ensure_parsed_docstring(obj: model.Documentable) -> Optional[model.Documenta
 class ParsedStanOnly(ParsedDocstring):
     """
     A L{ParsedDocstring} directly constructed from stan, for caching purposes.
-    
-    L{to_stan} method simply returns back what's given to L{ParsedStanOnly.__init__}. 
+
+    L{to_stan} method simply returns back what's given to L{ParsedStanOnly.__init__}.
     """
     def __init__(self, stan: Tag):
         super().__init__(fields=[])
         self._fromstan = stan
+
+    @property
     def has_body(self) -> bool:
         return True
-    def to_stan(self, docstring_linker: Any, compact:bool=False) -> Tag:
+
+    def to_stan(self, docstring_linker: Any) -> Tag:
         return self._fromstan
+
     def to_node(self) -> Any:
         raise NotImplementedError()
 
 def _get_parsed_summary(obj: model.Documentable) -> Tuple[Optional[model.Documentable], ParsedDocstring]:
     """
-    Ensures that the L{model.Documentable.parsed_summary} attribute of a documentable is set to it's final value. 
+    Ensures that the L{model.Documentable.parsed_summary} attribute of a documentable is set to it's final value.
     Do not generate summary twice.
-    
+
     @returns: Tuple: C{source}, C{parsed docstring}
     """
     source = ensure_parsed_docstring(obj)
-    
+
     if obj.parsed_summary is not None:
         return (source, obj.parsed_summary)
 
@@ -640,7 +704,7 @@ def _get_parsed_summary(obj: model.Documentable) -> Tuple[Optional[model.Documen
         # Tell mypy that if we found a docstring, we also have its source.
         assert obj.parsed_docstring is not None
         summary_parsed_doc = obj.parsed_docstring.get_summary()
-    
+
     obj.parsed_summary = summary_parsed_doc
 
     return (source, summary_parsed_doc)
@@ -648,10 +712,9 @@ def _get_parsed_summary(obj: model.Documentable) -> Tuple[Optional[model.Documen
 def get_to_stan_error(e: Exception) -> ParseError:
     return ParseError(f"{e.__class__.__name__}: {e}", 0)
 
-def safe_to_stan(parsed_doc: ParsedDocstring, 
+def safe_to_stan(parsed_doc: ParsedDocstring,
                  linker: 'DocstringLinker',
-                 ctx: model.Documentable, 
-                 compact: bool,
+                 ctx: model.Documentable,
                  fallback: Callable[[List[ParseError], ParsedDocstring, model.Documentable], Tag],
                  report: bool = True,
                  section:str='docstring') -> Tag:
@@ -662,7 +725,6 @@ def safe_to_stan(parsed_doc: ParsedDocstring,
     @param parsed_doc: The L{ParsedDocstring} to "stanify".
     @param linker: The L{DocstringLinker} to use to resolve links.
     @param ctx: The documentable context to use to report errors, passed to the C{fallback} function.
-    @param compact: Whether the generated html should be compact.
     @param fallback: A callable that returns a fallback stan if the convertion failed.
         It can also be used to set some state on the documentable context.
         Signature::
@@ -671,7 +733,7 @@ def safe_to_stan(parsed_doc: ParsedDocstring,
     @param section: Used for error messages.
     """
     try:
-        stan = parsed_doc.to_stan(linker, compact=compact)
+        stan = parsed_doc.to_stan(linker)
     except Exception as e:
         errs = [get_to_stan_error(e)]
         stan = fallback(errs, parsed_doc, ctx)
@@ -687,6 +749,36 @@ def format_docstring_fallback(errs: List[ParseError], parsed_doc:ParsedDocstring
         stan = parsed_doc_plain.to_stan(ctx.docstring_linker)
     return stan
 
+def _wrap_in_paragraph(body:Sequence["Flattenable"]) -> bool:
+    """
+    Whether to wrap the given docstring stan body inside a paragraph. 
+    """
+    has_paragraph = False
+    for e in body:
+        if isinstance(e, Tag) and e.tagName == 'p':
+            has_paragraph = True
+        # only check the first element of the body
+        break
+    return bool(len(body)>0 and not has_paragraph)
+
+def unwrap_docstring_stan(stan:Tag) -> "Flattenable":
+    """
+    Unwrap the body of the given C{Tag} instance if it has a non-empty tag name and 
+    ensure there is at least one paragraph. 
+
+    @note: This is the counterpart of what we're doing in L{HTMLTranslator.should_be_compact_paragraph()}.
+        Since the L{HTMLTranslator} is generic for all parsed docstrings types, it always generates compact paragraphs.
+
+        But for docstrings, we want to have at least one paragraph for consistency.
+    """
+    if stan.tagName:
+        return stan
+    body = stan.children
+    if _wrap_in_paragraph(body):
+        return tags.p(*body)
+    else:
+        return body
+
 def format_docstring(obj: model.Documentable) -> Tag:
     """Generate an HTML representation of a docstring"""
 
@@ -697,13 +789,8 @@ def format_docstring(obj: model.Documentable) -> Tag:
         ret(tags.p(class_='undocumented')("Undocumented"))
     else:
         assert obj.parsed_docstring is not None, "ensure_parsed_docstring() did not do it's job"
-        stan = safe_to_stan(obj.parsed_docstring, source.docstring_linker, source, 
-                            compact=False, fallback=format_docstring_fallback)
-        
-        if stan.tagName:
-            ret(stan)
-        else:
-            ret(*stan.children)
+        stan = safe_to_stan(obj.parsed_docstring, source.docstring_linker, source, fallback=format_docstring_fallback)
+        ret(unwrap_docstring_stan(stan))
 
     fh = FieldHandler(obj)
     if isinstance(obj, model.Function):
@@ -729,13 +816,13 @@ def format_summary(obj: model.Documentable) -> Tag:
     source, parsed_doc = _get_parsed_summary(obj)
     if not source:
         source = obj
-
-    # Disallow same_page_optimization in order to make sure we're not
-    # breaking links when including the summaries on other pages.
-    with source.docstring_linker.disable_same_page_optimazation():
+    
+    # do not optimize url in order to make sure we're always generating full urls.
+    # avoids breaking links when including the summaries on other pages.
+    with source.docstring_linker.switch_context(None):
         # ParserErrors will likely be reported by the full docstring as well,
         # so don't spam the log, pass report=False.
-        stan = safe_to_stan(parsed_doc, source.docstring_linker, source, compact=True, report=False,
+        stan = safe_to_stan(parsed_doc, source.docstring_linker, source, report=False,
                 fallback=format_summary_fallback)
 
     return stan
@@ -755,7 +842,7 @@ def format_undocumented(obj: model.Documentable) -> Tag:
 
     tag: Tag = tags.span(class_='undocumented')
     if sub_objects_with_docstring_count:
-        
+
         kind = obj.kind
         assert kind is not None # if kind is None, object is invisible
         tag(
@@ -763,7 +850,7 @@ def format_undocumented(obj: model.Documentable) -> Tag:
             ', '.join(
                 f"{sub_objects_with_docstring_count[kind]}/{sub_objects_total_count[kind]} "
                 f"{format_kind(kind, plural=sub_objects_with_docstring_count[kind]>=2).lower()}"
-                
+
                 for kind in sorted(sub_objects_total_count, key=(lambda x:x.value))
                 ),
             " documented"
@@ -774,18 +861,27 @@ def format_undocumented(obj: model.Documentable) -> Tag:
 
 
 def type2stan(obj: model.Documentable) -> Optional[Tag]:
+    """
+    Get the formatted type of this attribute.
+    """
+    # Currently only used for Attribute childs.
     parsed_type = get_parsed_type(obj)
     if parsed_type is None:
         return None
     else:
-        return safe_to_stan(parsed_type, obj.docstring_linker, obj, compact=True, 
+        _linker = linker._AnnotationLinker(obj)
+        return safe_to_stan(parsed_type, _linker, obj,
             fallback=colorized_pyval_fallback, section='annotation')
 
 def get_parsed_type(obj: model.Documentable) -> Optional[ParsedDocstring]:
+    """
+    Get the type of this attribute as parsed docstring.
+    """
     parsed_type = obj.parsed_type
     if parsed_type is not None:
         return parsed_type
 
+    # Only Attribute instances have the 'annotation' attribute.
     annotation: Optional[ast.expr] = getattr(obj, 'annotation', None)
     if annotation is not None:
         return colorize_inline_pyval(annotation)
@@ -793,14 +889,14 @@ def get_parsed_type(obj: model.Documentable) -> Optional[ParsedDocstring]:
     return None
 
 def format_toc(obj: model.Documentable) -> Optional[Tag]:
-    # Load the parsed_docstring if it's not already done. 
+    # Load the parsed_docstring if it's not already done.
     ensure_parsed_docstring(obj)
 
     if obj.parsed_docstring:
         if obj.system.options.sidebartocdepth > 0:
             toc = obj.parsed_docstring.get_toc(depth=obj.system.options.sidebartocdepth)
             if toc:
-                return safe_to_stan(toc, obj.docstring_linker, obj, compact=True, report=False,
+                return safe_to_stan(toc, obj.docstring_linker, obj, report=False,
                     fallback=lambda _,__,___:BROKEN)
     return None
 
@@ -812,7 +908,7 @@ field_name_to_kind = {
     }
 
 
-def extract_fields(obj: model.Documentable) -> None:
+def extract_fields(obj: model.CanContainImportsDocumentable) -> None:
     """Populate Attributes for module/class variables using fields from
     that module/class's docstring.
     Must only be called for objects that have a docstring.
@@ -846,7 +942,7 @@ def extract_fields(obj: model.Documentable) -> None:
 
 def format_kind(kind: model.DocumentableKind, plural: bool = False) -> str:
     """
-    Transform a `model.DocumentableKind` Enum value to string. 
+    Transform a `model.DocumentableKind` Enum value to string.
     """
     names = {
         model.DocumentableKind.PACKAGE         : 'Package',
@@ -869,7 +965,7 @@ def format_kind(kind: model.DocumentableKind, plural: bool = False) -> str:
         model.DocumentableKind.TYPE_VARIABLE   : 'Type Variable',
     }
     plurals = {
-        model.DocumentableKind.CLASS           : 'Classes', 
+        model.DocumentableKind.CLASS           : 'Classes',
         model.DocumentableKind.PROPERTY        : 'Properties',
         model.DocumentableKind.TYPE_ALIAS      : 'Type Aliases',
     }
@@ -891,12 +987,12 @@ def _format_constant_value(obj: model.Attribute) -> Iterator["Flattenable"]:
     row(tags.td(class_="fieldName")("Value"))
     # yield the first row.
     yield row
-    
-    doc = colorize_pyval(obj.value, 
+
+    doc = colorize_pyval(obj.value,
         linelen=obj.system.options.pyvalreprlinelen,
         maxlines=obj.system.options.pyvalreprmaxlines)
-    
-    value_repr = safe_to_stan(doc, obj.docstring_linker, obj, compact=True, 
+
+    value_repr = safe_to_stan(doc, obj.docstring_linker, obj,
         fallback=colorized_pyval_fallback, section='rendering of constant')
 
     # Report eventual warnings. It warns when a regex failed to parse.
@@ -924,7 +1020,7 @@ def _split_indentifier_parts_on_case(indentifier:str) -> List[str]:
     assert match is not None # the regex always matches
     prefix, text, suffix = match.groups(default='')
     text_parts = []
-    
+
     if text.islower() or text.isupper():
         # We assume snake_case or SCREAMING_SNAKE_CASE.
         text_parts = split(text, '_')
@@ -939,16 +1035,16 @@ def _split_indentifier_parts_on_case(indentifier:str) -> List[str]:
             if c.isupper() and not previous_was_upper:
                 text_parts.append(current_part)
                 current_part = ''
-            
+
             current_part += c
             previous_was_upper = c.isupper()
-        
+
         if current_part:
             text_parts.append(current_part)
 
     if not text_parts: # the name is composed only by underscores
         text_parts = ['']
-    
+
     if prefix:
         text_parts[0] = prefix + text_parts[0]
     if suffix:
@@ -967,7 +1063,7 @@ def insert_break_points(text: str) -> 'Flattenable':
 
     # We use tags.wbr instead of zero-width spaces because
     # zero-width spaces can interfer in subtle ways when copy/pasting a name.
-    
+
     r: List['Flattenable'] = []
     parts = text.split('.')
     for i,t in enumerate(parts):
@@ -980,3 +1076,70 @@ def insert_break_points(text: str) -> 'Flattenable':
             r += [tags.wbr(), '.']
     return tags.transparent(*r)
 
+def format_constructor_short_text(constructor: model.Function, forclass: model.Class) -> str:
+    """
+    Returns a simplified signature of the constructor.
+    C{forclass} is not always the function's parent, it can be a subclass.
+    """
+    args = ''
+    # for signature with more than 5 parameters, 
+    # we just show the elipsis after the fourth parameter
+    annotations = constructor.annotations.items()
+    many_param = len(annotations) > 6
+    
+    for index, (name, ann) in enumerate(annotations):
+        if name=='return':
+            continue
+
+        if many_param and index > 4:
+            args += ', ...'
+            break
+        
+        # Special casing __new__ because it's actually a static method
+        if index==0 and (constructor.name in ('__new__', '__init__') or 
+                         constructor.kind is model.DocumentableKind.CLASS_METHOD):
+            # Omit first argument (self/cls) from simplified signature.
+            continue
+        star = ''
+        if isinstance(name, VariableArgument):
+            star='*'
+        elif isinstance(name, KeywordArgument):
+            star='**'
+        
+        if args:
+            args += ', '
+        
+        args += f"{star}{name}"
+    
+    # display innner classes with their name starting at the top level class.
+    _current:model.CanContainImportsDocumentable = forclass
+    class_name = [] 
+    while isinstance(_current, model.Class):
+        class_name.append(_current.name)
+        _current = _current.parent
+    
+    callable_name = '.'.join(reversed(class_name))
+
+    if constructor.name not in ('__new__', '__init__'):
+        # We assume that the constructor is a method accessible in the Class.
+
+        callable_name += f'.{constructor.name}'
+
+    return f"{callable_name}({args})"
+
+def populate_constructors_extra_info(cls:model.Class) -> None:
+    """
+    Adds an extra information to be rendered based on Class constructors.
+    """
+    from pydoctor.templatewriter import util
+    constructors = cls.public_constructors
+    if constructors:
+        plural = 's' if len(constructors)>1 else ''
+        extra_epytext = f'Constructor{plural}: '
+        for i, c in enumerate(sorted(constructors, key=util.objects_order)):
+            if i != 0:
+                extra_epytext += ', '
+            short_text = format_constructor_short_text(c, cls)
+            extra_epytext += '`%s <%s>`' % (short_text, c.fullName())
+        
+        cls.extra_info.append(parse_docstring(cls, extra_epytext, cls, 'restructuredtext', section='constructor extra'))
