@@ -8,7 +8,6 @@ Forked from ``sphinx.ext.napoleon.docstring``.
     :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-import ast
 import collections
 from enum import Enum, auto
 import re
@@ -75,19 +74,16 @@ def is_obj_identifier(string: str) -> bool:
     An object identifier is a valid type string.
     But a valid type can be more complex than an object identifier.
     """
+    # support detecting "dict-like" as an object type even 
+    # if dashes are not actually allowed to keep compatibility with
+    # upstream napoleon.
+    string = string.replace('-', '_')
+    
     if string.isidentifier() or _xref_regex.match(string):
         return True
-    try:
-        # We simply try to load the string object with AST, if it's working
-        # we consider the string as a valid Python object identifier.
-        # Let's say 2048 is the maximum number of caracters
-        # that we'll try to parse here since 99% of the time it will
-        # suffice, and we don't want to add too much of expensive processing.
-        ast.parse(string[:2048])
-    except SyntaxError:
-        return False
-    else:
+    if all([p.isidentifier() or not p for p in string.split('.')]):
         return True
+    return False
 
 
 def is_type(string: str) -> bool:
@@ -738,20 +734,26 @@ class GoogleDocstring:
         if lines:
 
             before_colon, colon, _descs = self._partition_multiline_field_on_colon(
-                lines, format_validator=is_type
-            )
+                lines, format_validator=is_type)
 
             _type = ""
             if _descs:
                 if colon:
-                     # If a colon is detected, this means that the type is explicitely defined
+                    # If a colon is detected, this means that the type is explicitely defined
                     _type = before_colon
                 else:
                     # multiline free form returns clause
                     _descs.insert(0, before_colon)
             else:
-                # single line free form returns clause
-                _descs = [before_colon]
+                if colon:
+                    # If a colon is detected, it's the last part of the return clause like::
+                    # Returns:
+                    #   Union[str, int]:
+                    # Fixes https://github.com/sphinx-doc/sphinx/issues/9932
+                    _type = before_colon
+                else:
+                    # single line free form returns clause
+                    _descs = [before_colon]
 
             _descs = self.__class__(_descs).lines()
             _name = ""
@@ -1165,10 +1167,7 @@ class GoogleDocstring:
         section = section.lower() + ('s' if not section.endswith('s') else "")
         fields = self._consume_returns_section()
         multi = len(fields) > 1
-        if multi:
-            use_rtype = False
-        else:
-            use_rtype = True
+        use_rtype = False if multi else True
 
         lines = []  # type: List[str]
         for f in fields:
@@ -1184,6 +1183,7 @@ class GoogleDocstring:
                     lines.extend(self._format_block(f":{section}: * ", field))
             else:
                 if any(field):
+                    # only add :returns: if there's something to say
                     lines.extend(self._format_block(f":{section}: ", field))
                 if f.type and use_rtype:
                     lines.extend([f":{section.rstrip('s')}type: {self._convert_type(f.type, lineno=f.lineno)}", ""])
