@@ -1,12 +1,14 @@
 """Badly named module that contains the driving code for the rendering."""
 
 
+import itertools
 from pathlib import Path
 from typing import IO, Iterable, Type, TYPE_CHECKING
 
 from pydoctor import model
+from pydoctor.extensions import zopeinterface
 from pydoctor.templatewriter import (
-    DOCTYPE, pages, summary, TemplateLookup, IWriter, StaticTemplate
+    DOCTYPE, pages, summary, search, TemplateLookup, IWriter, StaticTemplate
 )
 
 from twisted.python.failure import Failure
@@ -82,13 +84,19 @@ class TemplateWriter(IWriter):
 
     def writeSummaryPages(self, system: model.System) -> None:
         import time
-        for pclass in summary.summaryPages(system):
+        for pclass in itertools.chain(summary.summaryPages(system), search.searchpages):
             system.msg('html', 'starting ' + pclass.__name__ + ' ...', nonl=True)
             T = time.time()
             page = pclass(system=system, template_lookup=self.template_lookup)
             with self.build_directory.joinpath(pclass.filename).open('wb') as fobj:
                 flattenToFile(fobj, page)
             system.msg('html', "took %fs"%(time.time() - T), wantsnl=False)
+        
+        # Generate the searchindex.json file
+        system.msg('html', 'starting lunr search index ...', nonl=True)
+        T = time.time()
+        search.write_lunr_index(self.build_directory, system=system)
+        system.msg('html', "took %fs"%(time.time() - T), wantsnl=False)
 
         if len(system.root_names) == 1:
             # If there is just a single root module it is written to index.html to produce nicer URLs.
@@ -118,16 +126,24 @@ class TemplateWriter(IWriter):
         if not ob.isVisible:
             return
         pclass: Type[pages.CommonPage] = pages.CommonPage
-        for parent in ob.__class__.__mro__:
+        class_name = ob.__class__.__name__
+        
+        # Special case the zope interface custom renderer. 
+        # TODO: Find a better way of handling renderer customizations and get rid of ZopeInterfaceClassPage completely.
+        if class_name == 'Class' and isinstance(ob, zopeinterface.ZopeInterfaceClass):
+            class_name = 'ZopeInterfaceClass'
+        
+        try:
             # This implementation relies on 'pages.commonpages' dict that ties
             # documentable class name (i.e. 'Class') with the
             # page class used for rendering: pages.ClassPage
-            try:
-                pclass = pages.commonpages[parent.__name__]
-            except KeyError:
-                continue
-            else:
-                break
+            pclass = pages.commonpages[class_name]
+        except KeyError:
+            ob.system.msg(section="html", 
+                # This is typically only reached in tests, when rendering Functions or Attributes with this method.
+                msg=f"Could not find page class suitable to render object type: {class_name!r}, using CommonPage.", 
+                once=True, thresh=-2)
+        
         ob.system.msg('html', str(ob), thresh=1)
         page = pclass(ob=ob, template_lookup=self.template_lookup)
         self.written_pages += 1
