@@ -1,7 +1,8 @@
 """PyDoctor's test suite."""
 
+import contextlib
 from logging import LogRecord
-from typing import Iterable, TYPE_CHECKING, Optional, Sequence
+from typing import Iterable, TYPE_CHECKING, Iterator, Optional, Sequence
 import sys
 import pytest
 from pathlib import Path
@@ -91,7 +92,8 @@ class NotFoundLinker(DocstringLinker):
 
     def link_xref(self, target: str, label: "Flattenable", lineno: int) -> Tag:
         return tags.code(label)
-
-    def resolve_identifier(self, identifier: str) -> Optional[str]:
-        return None
+    
+    @contextlib.contextmanager
+    def switch_context(self, ob: Optional[model.Documentable]) -> Iterator[None]:
+        yield
         
\ No newline at end of file
