@@ -247,7 +247,7 @@ def test_summary(markup:str) -> None:
     ]
     for src, summary_text in cases:
         errors: List[ParseError] = []
-        pdoc = get_parser_by_name(markup)(dedent(src), errors, False)
+        pdoc = get_parser_by_name(markup)(dedent(src), errors)
         assert not errors
         assert pdoc.get_summary() == pdoc.get_summary() # summary is cached inside ParsedDocstring as well.
         assert flatten_text(pdoc.get_summary().to_stan(NotFoundLinker())) == summary_text
