@@ -1,5 +1,6 @@
 from io import BytesIO
-from typing import Callable, Union, Any, cast, TYPE_CHECKING
+import re
+from typing import Callable, Union, Any, cast, Type, TYPE_CHECKING
 import pytest
 import warnings
 import sys
@@ -13,9 +14,11 @@ from pydoctor.templatewriter import (FailedToCreateTemplate, StaticTemplate, pag
                                      HtmlTemplate, UnsupportedTemplateVersion, 
                                      OverrideTemplateNotAllowed)
 from pydoctor.templatewriter.pages.table import ChildTable
-from pydoctor.templatewriter.summary import isClassNodePrivate, isPrivate, moduleSummary
-from pydoctor.test.test_astbuilder import fromText
+from pydoctor.templatewriter.pages.attributechild import AttributeChild
+from pydoctor.templatewriter.summary import isClassNodePrivate, isPrivate, moduleSummary, ClassIndexPage
+from pydoctor.test.test_astbuilder import fromText, systemcls_param
 from pydoctor.test.test_packages import processPackage, testpackages
+from pydoctor.test.test_epydoc2stan import InMemoryInventory
 from pydoctor.test import CapSys
 from pydoctor.themes import get_themes
 
@@ -54,6 +57,12 @@ def getHTMLOf(ob: model.Documentable) -> str:
     wr._writeDocsForOne(ob, f)
     return f.getvalue().decode()
 
+def getHTMLOfAttribute(ob: model.Attribute) -> str:
+    assert isinstance(ob, model.Attribute)
+    tlookup = TemplateLookup(template_dir)
+    stan = AttributeChild(util.DocGetter(), ob, [], 
+        AttributeChild.lookup_loader(tlookup),)
+    return flatten(stan)
 
 def test_sidebar() -> None:
     src = '''
@@ -521,6 +530,31 @@ def test_isClassNodePrivate() -> None:
     assert not isClassNodePrivate(cast(model.Class, mod.contents['_BaseForPublic']))
     assert isClassNodePrivate(cast(model.Class, mod.contents['_BaseForPrivate']))
 
+@systemcls_param
+def test_format_function_def_overloads(systemcls: Type[model.System]) -> None:
+    mod = fromText("""
+        from typing import overload, Union
+        @overload
+        def parse(s: str) -> str:
+            ...
+        @overload
+        def parse(s: bytes) -> bytes:
+            ...
+        def parse(s: Union[str, bytes]) -> Union[str, bytes]:
+            pass
+        """, systemcls=systemcls)
+    func = mod.contents['parse']
+    assert isinstance(func, model.Function)
+    
+    # We intentionally remove spaces before comparing
+    overloads_html = stanutils.flatten_text(list(pages.format_overloads(func))).replace(' ','')
+    assert '''(s:str)->str:''' in overloads_html
+    assert '''(s:bytes)->bytes:''' in overloads_html
+
+    # Confirm the actual function definition is not rendered
+    function_def_html = stanutils.flatten_text(list(pages.format_function_def(func.name, func.is_async, func)))
+    assert function_def_html == ''
+
 def test_format_signature() -> None:
     """Test C{pages.format_signature}. 
     
@@ -530,9 +564,8 @@ def test_format_signature() -> None:
     def func(a:Union[bytes, str]=_get_func_default(str), b:Any=re.compile(r'foo|bar'), *args:str, **kwargs:Any) -> Iterator[Union[str, bytes]]:
         ...
     ''')
-    assert ("""(a=_get_func_default(<wbr></wbr>str), b=re.compile("""
-            """r<span class="rst-variable-quote">'</span>foo<span class="rst-re-op">|</span>"""
-            """bar<span class="rst-variable-quote">'</span>), *args, **kwargs)""") in flatten(pages.format_signature(cast(model.Function, mod.contents['func'])))
+    assert ("""(a:Union[bytes,str]=_get_func_default(str),b:Any=re.compile(r'foo|bar'),*args:str,**kwargs:Any)->Iterator[Union[str,bytes]]""") in \
+        stanutils.flatten_text(pages.format_signature(cast(model.Function, mod.contents['func']))).replace(' ','')
 
 def test_format_decorators() -> None:
     """Test C{pages.format_decorators}"""
@@ -686,24 +719,23 @@ def test_crash_xmlstring_entities(capsys:CapSys, processtypes:bool) -> None:
     getHTMLOf(mod.contents['C'])
     out = capsys.readouterr().out
     warnings = '''\
-test:2: bad docstring: SAXParseException: <unknown>:2:25: undefined entity
-test:25: bad signature: SAXParseException: <unknown>:1:88: undefined entity
-test:25: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
-test:17: bad rendering of decorators: SAXParseException: <unknown>:1:102: undefined entity
-test:21: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
-test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity
-test:8: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
-test:10: bad rendering of constant: SAXParseException: <unknown>:1:112: undefined entity
-test:14: bad docstring: SAXParseException: <unknown>:1:13: undefined entity
-test:36: bad rendering of class signature: SAXParseException: <unknown>:1:104: undefined entity
+test:2: bad docstring: SAXParseException: <unknown>.+ undefined entity
+test:25: bad signature: SAXParseException: <unknown>.+ undefined entity
+test:17: bad rendering of decorators: SAXParseException: <unknown>.+ undefined entity
+test:21: bad signature: SAXParseException: <unknown>.+ undefined entity
+test:30: bad docstring: SAXParseException: <unknown>.+ undefined entity
+test:8: bad annotation: SAXParseException: <unknown>:.+ undefined entity
+test:10: bad rendering of constant: SAXParseException: <unknown>.+ undefined entity
+test:14: bad docstring: SAXParseException: <unknown>.+ undefined entity
+test:36: bad rendering of class signature: SAXParseException: <unknown>.+ undefined entity
 '''.splitlines()
     
     # Some how the type processing get rid of the non breaking spaces, but it's more an implementation
     # detail rather than a fix for the bug.
     if processtypes is True:
-        warnings.remove('test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity')
+        warnings.remove('test:30: bad docstring: SAXParseException: <unknown>.+ undefined entity')
     
-    assert out == '\n'.join(warnings)+'\n'
+    assert re.match('\n'.join(warnings), out)
 
 @pytest.mark.parametrize('processtypes', [True, False])
 def test_crash_xmlstring_entities_rst(capsys:CapSys, processtypes:bool) -> None:
@@ -719,21 +751,68 @@ def test_crash_xmlstring_entities_rst(capsys:CapSys, processtypes:bool) -> None:
     getHTMLOf(mod.contents['C'])
     out = capsys.readouterr().out
     warn_str = '''\
-test:2: bad docstring: SAXParseException: <unknown>:1:13: undefined entity
-test:25: bad signature: SAXParseException: <unknown>:1:88: undefined entity
-test:25: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
-test:17: bad rendering of decorators: SAXParseException: <unknown>:1:102: undefined entity
-test:21: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
-test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity
-test:8: bad annotation: SAXParseException: <unknown>:1:104: undefined entity
-test:10: bad rendering of constant: SAXParseException: <unknown>:1:112: undefined entity
-test:14: bad docstring: SAXParseException: <unknown>:1:13: undefined entity
-test:36: bad rendering of class signature: SAXParseException: <unknown>:1:104: undefined entity
+test:2: bad docstring: SAXParseException: <unknown>.+ undefined entity
+test:25: bad signature: SAXParseException: <unknown>.+ undefined entity
+test:17: bad rendering of decorators: SAXParseException: <unknown>.+ undefined entity
+test:21: bad signature: SAXParseException: <unknown>.+ undefined entity
+test:30: bad docstring: SAXParseException: <unknown>.+ undefined entity
+test:8: bad annotation: SAXParseException: <unknown>.+ undefined entity
+test:10: bad rendering of constant: SAXParseException: <unknown>.+ undefined entity
+test:14: bad docstring: SAXParseException: <unknown>.+ undefined entity
+test:36: bad rendering of class signature: SAXParseException: <unknown>.+ undefined entity
 '''
     warnings = warn_str.splitlines()
 
     if processtypes is True:
-        warnings.remove('test:30: bad docstring: SAXParseException: <unknown>:1:6: undefined entity')
+        warnings.remove('test:30: bad docstring: SAXParseException: <unknown>.+ undefined entity')
     
-    assert out == '\n'.join(warnings)+'\n'
+    assert re.match('\n'.join(warnings), out)
+
+def test_constructor_renders(capsys:CapSys) -> None:
+    src = '''\
+    class Animal(object):
+        # pydoctor can infer the constructor to be: "Animal(name)"
+        def __new__(cls, name):
+            ...
+    '''
+
+    mod = fromText(src)
+    html = getHTMLOf(mod.contents['Animal'])
+    assert 'Constructor: ' in html
+    assert 'Animal(name)' in html
+
+def test_typealias_string_form_linked() -> None:
+    """
+    The type aliases should be unstring before beeing presented to reader, such that
+    all elements can be linked. 
+    
+    Test for issue https://github.com/twisted/pydoctor/issues/704
+    """
+    
+    mod = fromText('''
+    from typing import Callable
+    ParserFunction = Callable[[str, List['ParseError']], 'ParsedDocstring']
+    class ParseError:
+        ...
+    class ParsedDocstring:
+        ...
+    ''', modname='pydoctor.epydoc.markup')
+
+    typealias = mod.contents['ParserFunction']
+    assert isinstance(typealias, model.Attribute)
+    html = getHTMLOfAttribute(typealias)
+    assert 'href="pydoctor.epydoc.markup.ParseError.html"' in html
+    assert 'href="pydoctor.epydoc.markup.ParsedDocstring.html"' in html
+
+def test_class_hierarchy_links_top_level_names() -> None:
+    system = model.System()
+    system.intersphinx = InMemoryInventory() # type:ignore
+    src = '''\
+    from socket import socket
+    class Stuff(socket):
+        ...
+    '''
+    mod = fromText(src, system=system)
+    index = flatten(ClassIndexPage(mod.system, TemplateLookup(template_dir)))
+    assert 'href="https://docs.python.org/3/library/socket.html#socket.socket"' in index
 
